#-------------------------------------------------------------------------------
#
#   Generic Mod Makefile - MC 1.7
#
#-------------------------------------------------------------------------------

#MODNAME := MyMod
#MODTITLE := My Mod
#MODSUBPKG := mymod

#MAJVER := 1
#MINVER := 0
#BUGVER := 0

#APIS := ic2

BACKUP_VOLNAME ?= GREGTHUMB5
BACKUP_DIRECTORY ?= ProjectsD/Minecraft

SERVER_SIDE ?= True

MC ?= 1.7.2

MODVER := $(MAJVER).$(MINVER).$(BUGVER)
MODPKG := gcewing/$(MODSUBPKG)
MODPKGNAME := gcewing.$(MODSUBPKG)
ASSETDIR := $(subst /,_,$(MODPKG))

MCP := mcp

INSTALLATION := /Local/Games/Minecraft/Profiles/1.6
PROFILE ?= ModTesting-1.7.2
SERVER ?= /Local/Games/Minecraft/Servers/$(MC)

TEST_PROFILE ?= plain
TEST_GAMEDIR ?= game/$(TEST_PROFILE)
TEST_MODDIR := $(TEST_GAMEDIR)/mods
PROFILE_MODDIR := $(INSTALLATION)/profiles/$(PROFILE)/mods
SERVER_MODDIR := $(SERVER)/mods

export MODNAME MODTITLE MODPKG MODSUBPKG MAJVER MINVER BUGVER MODVER

PROJECT := $(MODNAME)
BACKUP_VOL := /Volumes/$(BACKUP_VOLNAME)
BACKUP_SUBDIR := $(BACKUP_DIRECTORY)/$(PROJECT)
BACKUP_DIR := $(BACKUP_VOL)/$(BACKUP_SUBDIR)
BACKUP_EXCLUDE := --exclude build --exclude jars --exclude lib --exclude Releases --exclude Test \
	--exclude Ancillary --exclude game

SRC_INCLUDE := doc Makefile README.html src Tools

CPATH := $(MCP)/build/tmp/recompCls:`cd $(MCP); make cpath`
SPATH := src/mod:src/base:lib
CDEST := build/$(MC)
#JDEST := $(MCP)/game/mods/$(MODNAME).jar
JDEST := $(TEST_MODDIR)/$(MODNAME).jar

MANIFEST ?= src/mod/$(MODPKG)/MANIFEST.MF

#IC2PKG := ic2

RELJAR := $(MODNAME)-$(MODVER)-mc$(MC).jar
TSTJAR := build/libs/$(RELJAR)
RELZIP := $(MODNAME)-$(MODVER)-mc$(MC).zip
RELDOC := $(MODNAME)-$(MODVER)-Doc.zip
RELSRC := $(MODNAME)-$(MODVER)-mc$(MC)-Source.zip
WEBDIR := web/minecraft/mods/$(MODNAME)

#LANGFILE := build/$(MODNAME).lang
#LANGDEST := src/resources/assets/`echo $(MODNAME) | tr '[:upper:]' '[:lower:]'`/lang

#JFLAGS = -Xlint:deprecation
MCUSER = gcewing

JC := javac $(JFLAGS)

JARCONTENTS ?= \
    -C $(1) $(MODPKG) \
    -C src/mod assets \
    $(EXTRA_JARCONTENTS)

BUILDJAR = \
	for item in `cd build/$(MC); find . -name '*.class'`; do \
		args="$$args -C $(2) $$item"; \
	done; \
	for item in `cd src/resources; echo *`; do \
		args="$$args -C src/resources $$item"; \
	done; \
	jar cf $(1) $$args; \
	if [ -e $(MANIFEST) ]; then \
		jar ufm $(1) $(MANIFEST); \
	fi

cpath:
	@echo $(CPATH)

#----------------------------------------------------------------------------------

test: compile install run

#test_jar: compile install jar install_jar run_jar
test_jar: compile_jar install_jar run_jar

#test_server: compile install jar install_jar run_server

#release: compile install jar relpkg
release: compile_jar relpkg

#-----------------------------------------------------------------------------------------------

gradle.properties: Makefile Tools/Mod.make
	(echo version = $(MODVER)-mc$(MC); \
	 echo group = $(MODPKGNAME); \
	 echo jarName = $(MODNAME)) > $@

#-----------------------------------------------------------------------------------------------

src_dirs:
	mkdir -p src/base/$(MODPKG) src/mod/$(MODPKG)
	rm -f src-base src-gcewing
	ln -s src/base/$(MODPKG) src-base
	ln -s src/mod/$(MODPKG) src-gcewing

texture_dirs:
	mkdir -p src/resources/assets/$(ASSETDIR)/textures/blocks
	mkdir -p src/resources/assets/$(ASSETDIR)/textures/items

texture_links:
	#rm -f textures-blocks textures-items
	rm -f textures
	#ln -s src/resources/assets/$(ASSETDIR)/textures/blocks textures-blocks
	#ln -s src/resources/assets/$(ASSETDIR)/textures/items textures-items
	ln -s src/resources/assets/$(ASSETDIR)/textures .

#-----------------------------------------------------------------------------------------------

compile: $(CDEST) info m4base lang
	@echo Compiling
	@rm -rf $(CDEST)/*
	@$(JC) -classpath $(CPATH) -sourcepath $(SPATH) -d $(CDEST) `find lib src -name '*.java'`

lang:
	Tools/genlang $(MODNAME) $(MODPKGNAME) src/$(MODNAME).names

#lang:
#	if [ -e $(LANGFILE) ]; then \
#		mkdir -p $(LANGDEST); \
#		cp $(LANGFILE) $(LANGDEST)/en_US.lang; \
#	fi

$(CDEST):
	mkdir -p $(CDEST)

test_info:
	Tools/geninfo

info:
	Tools/geninfo > src/mod/$(MODPKG)/Info.java

m4base:
	@echo Preprocessing base
	@(cd src/base/$(MODPKG); \
	 for f in *.java; do \
	 	m4 -P -DMOD_PACKAGE=$(MODPKGNAME) < ../../../../../Base/$$f > $$f; \
	 done)

install:
	mkdir -p $(TEST_MODDIR)
	rm -f $(JDEST)
	$(call BUILDJAR, $(JDEST), build/$(MC))

run:
	export GAMEDIR=`pwd`/$(TEST_GAMEDIR); cd $(MCP); make run

#run:
#	export LANGDUMP=$(MODNAME) LANGFILE=`pwd`/build/$(MODNAME).lang; cd $(MCP); make run

#run:
#	cd $(MCP); ./startclient.sh $(RUNARGS) $(MCUSER)

#-----------------------------------------------------------------------------------------------

compile_jar: gradle.properties info m4base
	gradle build

#jar: reob reljar
#
#reob: reob_prep reob_run reob_cleanup
#
#reob_prep:
#	for d in `(cd build/$(MC); find . -type d)`; do \
#		mkdir -p $(MCP)/bin/minecraft/$$d; \
#	done
#	for f in `(cd build/$(MC); find . -name '*.class')`; do \
#		cp build/$(MC)/$$f $(MCP)/bin/minecraft/$$f; \
#	done
#
#reob_run:
#	cd $(MCP); ./reobfuscate_srg.sh --client
#
#reob_cleanup:
#	for f in `(cd build/$(MC); find $(MODPKG) -name '*.class')`; do \
#		rm -f $(MCP)/bin/minecraft/$$f; \
#	done

#reljar:
#	$(call BUILDJAR, $(RELJAR), $(MCP)/reobf/minecraft)
#	unzip -l $(RELJAR)

#-----------------------------------------------------------------------------------------------

install_jar:
		rm -f $(PROFILE_MODDIR)/$(MODNAME)-*
		cp $(TSTJAR) $(PROFILE_MODDIR)
ifeq ($(SERVER_SIDE), True)
		rm -f $(SERVER_MODDIR)/$(MODNAME)-*
		cp $(TSTJAR) $(SERVER_MODDIR)
endif

#-----------------------------------------------------------------------------------------------

run_jar:
	Tools/mclaunch $(INSTALLATION) -p $(PROFILE) -u $(MCUSER)

run_server:
	cd $(SERVER); start

#-----------------------------------------------------------------------------------------------

relpkg: reljar reldoc relsrc

reljar:
	cp build/libs/$(RELJAR) .

reldoc: build/README.html
	rm -f $(RELDOC)
	cd build; zip -r ../$(RELDOC) README.html
	zip -r $(RELDOC) doc

build/%.html: %.html
	m4 -P -DMODVER=$(MODVER) -DMCVER=mc$(MC) $< > $@

relsrc:
	rm -f $(RELSRC)
	zip -r $(RELSRC) $(SRC_INCLUDE)
	
#----------------------------------------------------------------------------------

upload:
	rsync --progress --times $(RELJAR) $(RELDOC) $(RELSRC) $(COSC):$(WEBDIR)/download
	ssh $(COSC) 'cd $(WEBDIR); unzip -o download/$(RELDOC)'

#----------------------------------------------------------------------------------

backup: backup_subdir
	mkdir -p Archive
	cp -r ../Tools Archive
	DATE=`date +%Y-%m-%d`; \
	TARFILE=$(BACKUP_DIR)/$(PROJECT)-$$DATE-mc$(MC).tar.gz; \
	tar zcvf $$TARFILE $(BACKUP_EXCLUDE) * && \
	ls -l $$TARFILE
	df -H $(BACKUP_VOL)

backup_subdir:
	cd $(BACKUP_VOL); mkdir -p $(BACKUP_SUBDIR)
